@@ -9,6 +9,7 @@ import socket
 import subprocess
 import time
 from typing import Any, Dict, List, Optional, Union
+import tempfile
 
 from IPython.core.magic import Magics, cell_magic, line_magic, magics_class
 from agate import Table
@@ -28,15 +29,18 @@ class ServerProcess():
         self.cmd = ['dbt'] + cmd_args
         parser = argparse.ArgumentParser()
         parser.add_argument('--port', default=8580, type=int)
+        parser.add_argument('--existing', action='store_true')
         args, _ = parser.parse_known_args(cmd_args[1:]) #remove rpc argument
         self.port = args.port
+        self.existing = args.existing
 
     def run(self):
         self.proc = subprocess.Popen(self.cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
         return self.proc
 
     def start(self):
-        self.run()
+        if not self.existing:
+            self.run()
         for _ in range(30):
             if self.is_up():
                 break
@@ -61,7 +65,8 @@ class ServerProcess():
             sock.connect(('localhost', self.port))
         except socket.error:
             return False
-        sock.close()
+        finally:
+            sock.close()
         return True
 
     def _compare_result(self, result):
@@ -361,9 +366,12 @@ class Querier:
         if id is not None:
             assert data['id'] == id
         assert data['jsonrpc'] == '2.0'
-        assert 'result' in data
-        assert 'error' not in data
-        return data['result']
+        if 'error' in data or 'result' not in data:
+            raise RuntimeError(
+                'Got invalid response: {}'.format(data)
+            )
+        else:
+            return data['result']
 
     def is_async_result(self, data: Dict[str, Any], id=None) -> str:
         result = self.is_result(data, id)
@@ -411,6 +419,7 @@ class DBTMagics(Magics):
         # You must call the parent constructor
         super(DBTMagics, self).__init__(shell)
         atexit.register(self._stop)
+        self.last_result = None
 
     def _stop(self):
         try:
@@ -425,12 +434,17 @@ class DBTMagics(Magics):
             self._stop()
             proc = ServerProcess(args)
             self.querier = Querier(proc)
-            self.querier.server.start()
+            try:
+                self.querier.server.start()
+            except Exception:
+                return self.querier.server.proc.stdout.read().decode('utf-8')
+
             return(self.querier)
 
     @cell_magic
     def run_sql(self, line, cell):
-        query = cell
+        max_rows = 500
+        query = "select * from ({}) dbt_run_sql limit {}".format(cell, max_rows)
         resp = self.querier.run_sql(query)
         result = self.querier.async_wait_for_result(resp)
         resp_table = result['results'][0]['table']
@@ -438,6 +452,7 @@ class DBTMagics(Magics):
         rows_dict = [dict(zip(column_names, row)) for row in resp_table['rows']]
         table_result = Table.from_object(rows_dict)
         table_result.print_table()
+        self.last_result = table_result
         return table_result
 
     @cell_magic
@@ -446,4 +461,19 @@ class DBTMagics(Magics):
         resp = self.querier.compile_sql(query)
         result = self.querier.async_wait_for_result(resp)
         compiled_sql = result['results'][0]['compiled_sql']
+        self.last_result = compiled_sql
         return Sql(compiled_sql)
+
+    @line_magic
+    def dbt_clipboard(self, line):
+        import pyperclip
+        if isinstance(self.last_result, Table):
+            try:
+                fd, path = tempfile.mkstemp()
+                self.last_result.to_csv(path)
+                with open(path) as f:
+                    pyperclip.copy(f.read())
+            finally:
+                os.remove(path)
+        else:
+            pyperclip.copy(self.last_result)
